import * as monaco from 'monaco-editor/esm/vs/editor/editor.api';
import 'monaco-editor/esm/vs/language/html/monaco.contribution';
import { DisposableStore } from 'vscode/monaco';
import { StandaloneServices, ICodeEditorService } from 'vscode/services';
import { registerWorkerLoader } from '../worker.js';
import { registerConfigurations, getConfiguration, onConfigurationChanged } from '../configuration/index.js';

registerWorkerLoader('html', () => new Worker(new URL('monaco-editor/esm/vs/language/html/html.worker', import.meta.url)));
registerWorkerLoader('handlebars', () => new Worker(new URL('monaco-editor/esm/vs/language/html/html.worker', import.meta.url)));
registerWorkerLoader('razor', () => new Worker(new URL('monaco-editor/esm/vs/language/html/html.worker', import.meta.url)));
/**
 * Start autoclosing html tags
 * Inspired by https://github.com/microsoft/monaco-editor/issues/221
 * and https://github.com/microsoft/vscode/blob/5943dac8ba32df1963a1760c660b3f16f5fcdc51/extensions/html-language-features/client/src/autoInsertion.ts
 */
registerConfigurations([{
        id: 'html',
        order: 20,
        type: 'object',
        title: 'HTML',
        properties: {
            'html.autoClosingTags': {
                type: 'boolean',
                scope: 4 /* ConfigurationScope.RESOURCE */,
                default: true,
                description: 'Enable/disable autoclosing of HTML tags.'
            }
        }
    }]);
let autoClosingTags = false;
setTimeout(() => {
    // In a timeout so the service can be overriden
    autoClosingTags = getConfiguration(undefined, 'html.autoClosingTags');
    onConfigurationChanged(e => {
        if (e.affectsConfiguration('html.autoClosingTags')) {
            autoClosingTags = getConfiguration(undefined, 'html.autoClosingTags');
        }
    });
});
function autoCloseHtmlTags(editor) {
    const disposableStore = new DisposableStore();
    let timeout;
    disposableStore.add({
        dispose: () => {
            if (timeout != null) {
                window.clearTimeout(timeout);
            }
        }
    });
    disposableStore.add(editor.onDidChangeModelContent(e => {
        if (timeout != null) {
            window.clearTimeout(timeout);
            timeout = undefined;
        }
        if (!autoClosingTags || e.isRedoing || e.isUndoing) {
            return;
        }
        const model = editor.getModel();
        if (model == null || model.getLanguageId() !== 'html') {
            return;
        }
        timeout = window.setTimeout(() => {
            for (const { range, rangeLength, text } of e.changes) {
                if (!text.endsWith('>')) {
                    continue;
                }
                const untilLine = model.getValueInRange({
                    startLineNumber: 1,
                    startColumn: 1,
                    endLineNumber: range.endLineNumber,
                    endColumn: range.endColumn + rangeLength + 1
                });
                const enclosingTag = untilLine.match(/.*<(\w+)>$/)?.[1];
                if (enclosingTag == null || enclosingTag.includes('/')) {
                    return;
                }
                const newRange = new monaco.Range(range.endLineNumber, range.endColumn + rangeLength + 1, range.endLineNumber, range.endColumn + rangeLength + enclosingTag.length + 1);
                const selections = editor.getSelections();
                const rest = model.getValueInRange(newRange);
                editor.executeEdits('autoclosing', [{
                        range: newRange,
                        text: `</${enclosingTag}>${rest}`,
                        forceMoveMarkers: true
                    }], selections ?? undefined);
            }
        }, 100);
    }));
    return disposableStore;
}
setTimeout(() => {
    // In a timeout so the service can be overriden
    const codeEditors = StandaloneServices.get(ICodeEditorService).listCodeEditors();
    for (const editor of codeEditors) {
        autoCloseHtmlTags(editor);
    }
    monaco.editor.onDidCreateEditor(editor => {
        autoCloseHtmlTags(editor);
    });
});
/**
 * End autoclosing html tags
 */
